package com.example.demo

import com.mongodb.ConnectionString
import com.mongodb.MongoClientSettings
import com.mongodb.ReadPreference
import com.mongodb.TransactionOptions
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.data.mongodb.ReactiveMongoDatabaseFactory
import org.springframework.data.mongodb.ReactiveMongoTransactionManager
import org.springframework.data.mongodb.config.AbstractReactiveMongoConfiguration
import org.springframework.data.mongodb.repository.config.EnableReactiveMongoRepositories
import java.util.concurrent.TimeUnit

@Configuration
@EnableReactiveMongoRepositories
class MongoConfiguration : AbstractReactiveMongoConfiguration() {

    override fun getDatabaseName(): String {
        return "anyname"
    }

    override fun mongoClientSettings(): MongoClientSettings {
        return MongoClientSettings.builder()
            .applyConnectionString(ConnectionString("mongodb://localhost:27017/anyname?replicaSet=rs0"))
            .applyToConnectionPoolSettings {
                it
                    .maxWaitTime(5000, TimeUnit.MILLISECONDS)
                    .maxSize(50)
            }
            .build()
    }

    @Bean
    fun transactionManager(
        factory: ReactiveMongoDatabaseFactory,
    ): ReactiveMongoTransactionManager {
        return ReactiveMongoTransactionManager(
            factory,
            TransactionOptions.builder().readPreference(ReadPreference.primaryPreferred()).build()
        )
    }
}
