package com.example.demo

import org.bson.Document
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.data.mongodb.core.ReactiveMongoTemplate
import org.springframework.stereotype.Component
import reactor.core.publisher.Mono

@Component
class MongoValidationSetupUtils {

    @Autowired
    private lateinit var mongoTemplate: ReactiveMongoTemplate

    private val collectionName = "any-document"

    fun setupValidationRule(id: String): Mono<Void> {
        val validator = Document("\$expr", Document("\$ne", listOf("\$_id", id)))
        val dbCommand = Document("collMod", collectionName)
            .append("validator", validator)
            .append("validationLevel", "strict")
            .append("validationAction", "error")

        return mongoTemplate.mongoDatabase
            .flatMap { db ->
                Mono.from(db.runCommand(dbCommand))
            }
            .then()
    }

    fun resetValidationRule(): Mono<Void> {
        val dbCommand = Document("collMod", collectionName)
            .append("validator", Document())
            .append("validationLevel", "off")
            .append("validationAction", "warn")

        return mongoTemplate.mongoDatabase
            .flatMap { db ->
                Mono.from(db.runCommand(dbCommand))
            }
            .then()
    }
}
